import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Table, Typography, Switch, Modal, Input, InputNumber, Tooltip, Button, notification, message, Select } from 'antd'
import { ExclamationCircleOutlined, EditOutlined, PlusOutlined, PlusCircleOutlined, DeleteOutlined } from '@ant-design/icons'
import moment from 'moment'

import Api from '@/api'
import asyncComponent from '@/utils/asyncComponent'
import Utils, { getEditTableSql, getMark } from '@/utils/utils.js'
import MkIcon from '@/components/mk-icon'
import MKEmitter from '@/utils/events.js'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import CusSwitch from './cusSwitch'
import Encrypts from '@/components/encrypts'
import './index.scss'

const { Paragraph } = Typography
const { confirm } = Modal
const CardCellComponent = asyncComponent(() => import('@/tabviews/custom/components/card/cardcellList'))

class BodyRow extends React.Component {
  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props.data), fromJS(nextProps.data))
  }

  render() {
    let { lineMarks, data, ...resProps } = this.props
    let style = {}
    let className = ''

    lineMarks && lineMarks.some(mark => {
      let originVal = data[mark.field[0]] + ''
      let contrastVal = ''
      let result = false

      if (mark.field[1] === 'static') {
        contrastVal = mark.contrastValue + ''
      } else {
        contrastVal = data[mark.field[2]] + ''
      }

      if (mark.match === '=') {
        result = originVal === contrastVal
      } else if (mark.match === '!=') {
        result = originVal !== contrastVal
      } else if (mark.match === 'like') {
        result = originVal.indexOf(contrastVal) > -1
      } else if (mark.match === '>') {
        try {
          originVal = parseFloat(originVal)
          contrastVal = parseFloat(contrastVal)
        } catch (e) {
          originVal = NaN
        }

        if (!isNaN(originVal) && !isNaN(contrastVal) && originVal > contrastVal) {
          result = true
        }
      } else if (mark.match === '<') {
        try {
          originVal = parseFloat(originVal)
          contrastVal = parseFloat(contrastVal)
        } catch (e) {
          originVal = NaN
        }

        if (!isNaN(originVal) && !isNaN(contrastVal) && originVal < contrastVal) {
          result = true
        }
      }

      if (result) {
        if (mark.signType[0] === 'font') {
          style.color = mark.color
        } else if (mark.signType[0] === 'background') {
          style.background = mark.color
          if (mark.fontColor) {
            style.color = mark.fontColor
          }
          className = 'background'
        } else if (mark.signType[0] === 'underline') {
          style.textDecoration = 'underline'
          style.color = mark.color
        } else if (mark.signType[0] === 'line-through') {
          style.textDecoration = 'line-through'
          style.color = mark.color
        }
      }

      return result
    })

    return <tr {...resProps} className={className} style={style}/>
  }
}

class BodyCell extends React.Component {
  state = {
    editing: false,
    err: null
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props.record), fromJS(nextProps.record)) ||
      nextState.editing !== this.state.editing ||
      nextState.err !== this.state.err
  }

  componentDidMount () {
    MKEmitter.addListener('tdFocus', this.tdFocus)
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('tdFocus', this.tdFocus)
  }

  tdFocus = (id) => {
    const { col, record } = this.props

    if (id !== col.uuid + record.$$uuid) return
    this.focus()
  }

  enterPress = () => {
    const { col, record } = this.props
    const { value } = this.state

    this.setState({editing: false})
    setTimeout(() => {
      if (col.enter === '$next') {
        MKEmitter.emit('nextLine', col, record.$$uuid)
      } else if (col.enter === '$sub') {
        MKEmitter.emit('subLine', col, record)
      } else if (col.enter !== '$noAct') {
        MKEmitter.emit('tdFocus', col.enter + record.$$uuid)
      }
    }, 50)

    if (value !== record[col.field]) {
      MKEmitter.emit('changeRecord', col.tableId, {...record, [col.field]: value})
    }
  }

  focus = () => {
    const { col, record } = this.props

    if (col.editType === 'switch' || col.editType === 'select') {
      this.setState({editing: true}, () => {
        let node = document.getElementById(col.uuid + record.$$uuid)
        node && node.click()
      })
    } else {
      let err = null
      let val = record[col.field] !== undefined ? record[col.field] : ''
  
      if (col.type === 'number') {
        val = +val
        if (isNaN(val)) {
          val = 0
        }
        if (typeof(col.max) === 'number' && val > col.max) {
          err = col.label + '最大为' + col.max
        } else if (typeof(col.min) === 'number' && val < col.min) {
          err = col.label + '最小为' + col.min
        }
      } else if (col.required === 'true' && !val) {
        err = '请填写' + col.label
      }
  
      this.setState({editing: true, value: val, err}, () => {
        let node = document.getElementById(col.uuid + record.$$uuid)
        node && node.select()
      })
    }
  }

  onBlur = () => {
    const { col, record } = this.props
    const { value } = this.state

    this.setState({editing: false})

    if (value !== record[col.field]) {
      MKEmitter.emit('changeRecord', col.tableId, {...record, [col.field]: value})
    }
  }
  
  onChange = (val) => {
    const { col } = this.props
    
    let err = null

    if (col.type === 'number') {
      val = +val
      if (isNaN(val)) {
        val = 0
      }
      if (typeof(col.max) === 'number' && val > col.max) {
        err = col.label + '最大为' + col.max
      } else if (typeof(col.min) === 'number' && val < col.min) {
        err = col.label + '最小为' + col.min
      }
    } else if (col.required === 'true' && !val) {
      err = '请填写' + col.label
    }
    this.setState({value: val, err})
  }

  onSwitchChange = (val, label) => {
    const { col, record } = this.props

    this.setState({editing: false})

    setTimeout(() => {
      if (col.enter === '$next') {
        MKEmitter.emit('nextLine', col, record.$$uuid)
      } else if (col.enter === '$sub') {
        MKEmitter.emit('subLine', col, record)
      } else if (col.enter !== '$noAct') {
        MKEmitter.emit('tdFocus', col.enter + record.$$uuid)
      }
    }, 50)

    let values = {}
    if (col.editField) {
      values[col.field] = label
      values[col.editField] = val
    } else {
      values[col.field] = val
    }

    MKEmitter.emit('changeRecord', col.tableId, {...record, ...values})
  }

  onSelectChange = (val, option) => {
    const { col, record } = this.props

    let values = {}
    let _option = col.options.filter(m => m.key === option.key)[0]

    if (_option) {
      if (col.linkSubField) {
        col.linkSubField.forEach(m => {
          values[m] = _option[m] !== undefined ? _option[m] : ''
        })
      }

      if (col.editField) {
        values[col.field] = _option.label
        values[col.editField] = val
      } else {
        values[col.field] = val
      }
    }

    this.setState({editing: false})

    setTimeout(() => {
      if (col.enter === '$next') {
        MKEmitter.emit('nextLine', col, record.$$uuid)
      } else if (col.enter === '$sub') {
        MKEmitter.emit('subLine', col, record)
      } else if (col.enter !== '$noAct') {
        MKEmitter.emit('tdFocus', col.enter + record.$$uuid)
      }
    }, 50)

    MKEmitter.emit('changeRecord', col.tableId, {...record, ...values})
  }

  switchBlur = () => {
    setTimeout(() => {
      this.setState({editing: false})
    }, 10)
  }

  render() {
    let { col, config, record, style, className, ...resProps } = this.props
    const { editing, value, err } = this.state

    if (!col) return (<td {...resProps} className={className} style={style}/>)

    let children = null
    if (col.type === 'text') {
      let content = ''
      if (record[col.field] !== undefined) {
        content = `${record[col.field]}`
      }

      if (content !== '') {
        if (col.textFormat === 'YYYY-MM-DD' && /^[1-9]\d{3}(-|\/)(0[1-9]|1[0-2])(-|\/)(0[1-9]|[1-2][0-9]|3[0-1])/.test(content)) {
          content = `${content.substr(0, 4)}-${content.substr(5, 2)}-${content.substr(8, 2)}`
        } else if (col.textFormat === 'YYYY-MM-DD HH:mm:ss' && /^[1-9]\d{3}(-|\/)(0[1-9]|1[0-2])(-|\/)(0[1-9]|[1-2][0-9]|3[0-1]).([0-1][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]/.test(content)) {
          content = `${content.substr(0, 4)}-${content.substr(5, 2)}-${content.substr(8, 2)} ${content.substr(11, 2)}:${content.substr(14, 2)}:${content.substr(17, 2)}`
        } else if (col.textFormat === 'encryption') {
          content = <span>{col.prefix || ''}<Encrypts value={content} />{col.postfix || ''}</span>
        }

        if (col.textFormat !== 'encryption') {
          content = (col.prefix || '') + content + (col.postfix || '')
        }
      }

      if (col.marks) {
        let mark = getMark(col.marks, record, style)

        style = mark.style

        if (mark.icon) {
          if (mark.position === 'front') {
            content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
          } else {
            content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
          }
        } else if (mark.innerStyle) {
          content = <span style={mark.innerStyle}>{content}</span>
        }
      }

      if (col.editable === 'true') {
        if (editing) {
          if (!col.editType || col.editType === 'text') {
            return (<td className="editing_table_cell">
              <Input id={col.uuid + record.$$uuid} defaultValue={value} onChange={(e) => this.onChange(e.target.value)} onPressEnter={this.enterPress} onBlur={this.onBlur}/>
              {err ? <Tooltip title={err}><ExclamationCircleOutlined /></Tooltip> : null}
            </td>)
          } else if (col.editType === 'switch') {
            let _value = ''
            if (col.editField) {
              _value = record[col.editField] !== undefined ? record[col.editField] : ''
            } else {
              _value = record[col.field] !== undefined ? record[col.field] : ''
            }
            return (<td className="editing_table_cell">
              <CusSwitch config={col} defaultValue={_value} autoFocus={true} onChange={this.onSwitchChange} onBlur={this.switchBlur}/>
            </td>)
          } else {
            let _value = ''
            if (col.editField) {
              _value = record[col.editField] !== undefined ? record[col.editField] : ''
            } else {
              _value = record[col.field] !== undefined ? record[col.field] : ''
            }
            return (<td className="editing_table_cell">
              <Select
                showSearch
                defaultValue={_value}
                dropdownClassName="edit-table-dropdown"
                dropdownMatchSelectWidth={col.dropdown === 'fixed'}
                id={col.uuid + record.$$uuid}
                onBlur={() => this.setState({editing: false})}
                filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
                onSelect={this.onSelectChange}
              >
                {col.options.map((item, i) => (<Select.Option key={item.key} disabled={item.$disabled} value={item.value}>{item.label}</Select.Option>))}
              </Select>
            </td>)
          }
        } else {
          return (<td className={className + ' pointer'} style={style}>
            {col.addable ? <PlusCircleOutlined onClick={() => MKEmitter.emit('addRecord', col.tableId, {...record})} className="mk-editable mk-plus"/> : null}
            <div className="mk-mask" onClick={this.focus}></div>{content}
            {col.delable ? <DeleteOutlined onClick={() => MKEmitter.emit('delRecord', col.tableId, {...record})} className="mk-editable mk-del"/> : null}
          </td>)
        }
      } else {
        children = content
      }
    } else if (col.type === 'number') {
      let content = ''
      try {
        content = parseFloat(record[col.field])
        if (isNaN(content)) {
          content = ''
        }
      } catch (e) {
        content = ''
      }

      if (content !== '') {
        if (col.round) {
          content = Math.round(content * col.round) / col.round
        }
        if (col.format === 'percent') {
          content = content * 100
        } else if (col.format === 'abs') {
          content = Math.abs(content)
        }
        if (col.round) {
          content = content.toFixed(col.decimal)
        }
  
        if (col.format === 'thdSeparator') {
          content = content.replace(/\d{1,3}(?=(\d{3})+(\.\d*)?$)/g, '$&,')
        }
  
        content = (col.prefix || '') + content + (col.postfix || '')
      }

      if (col.marks) {
        let mark = getMark(col.marks, record, style)

        style = mark.style

        if (mark.icon) {
          if (mark.position === 'front') {
            content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
          } else {
            content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
          }
        } else if (mark.innerStyle) {
          content = <span style={mark.innerStyle}>{content}</span>
        }
      }

      if (col.editable === 'true') {
        if (editing) {
          return (<td className="editing_table_cell">
            <InputNumber id={col.uuid + record.$$uuid} defaultValue={value} onChange={(val) => this.onChange(val)} onPressEnter={this.enterPress} onBlur={this.onBlur}/>
            {err ? <Tooltip title={err}><ExclamationCircleOutlined /></Tooltip> : null}
          </td>)
        } else {
          return (<td className={className + ' pointer'} style={style}>
            {col.addable ? <PlusCircleOutlined onClick={() => MKEmitter.emit('addRecord', col.tableId, {...record})} className="mk-editable mk-plus"/> : null}
            <div className="mk-mask" onClick={this.focus}></div>{content}
            {col.delable ? <DeleteOutlined onClick={() => MKEmitter.emit('delRecord', col.tableId, {...record})} className="mk-editable mk-del"/> : null}
          </td>)
        }
      } else {
        children = content
      }
    } else if (col.type === 'textarea') {
      let content = ''
      if (record[col.field] !== undefined) {
        content = `${record[col.field]}`
      }

      if (content) {
        content = (col.prefix || '') + content + (col.postfix || '')
      }

      children = (
        <div>
          {content ? <Paragraph copyable ellipsis={{ rows: 3, expandable: true }}>{content}</Paragraph> : null }
        </div>
      )
    } else if (col.type === 'formula') {
      let content = col.formula
      Object.keys(record).forEach(key => {
        let reg = new RegExp('@' + key + '@', 'ig')
        content = content.replace(reg, record[key])
      })

      if (col.eval !== 'false') {
        try {
          // eslint-disable-next-line
          content = eval(content)
        } catch (e) {
          content = ''
        }
      }

      content = content === undefined ? '' : content

      if (content !== '') {
        content = `${col.prefix || ''}${content}${col.postfix || ''}`

        if (col.eval === 'false') {
          content = content.replace(/\n/ig, '<br/>').replace(/\s/ig, '&nbsp;')
          content = <span dangerouslySetInnerHTML={{__html: content}}></span>
        }
      }

      if (col.marks) {
        let mark = getMark(col.marks, record, style)

        style = mark.style

        if (mark.icon) {
          if (mark.position === 'front') {
            content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
          } else {
            content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
          }
        } else if (mark.innerStyle) {
          content = <span style={mark.innerStyle}>{content}</span>
        }
      }

      children = content
    } else if (col.type === 'custom') {
      style.padding = '0px'
      if (col.style) {
        style = {...style, ...col.style}
      }

      children = (
        <CardCellComponent data={record} cards={config} elements={col.elements}/>
      )
    } else if (col.type === 'action') {
      style.padding = '0px'
      if (col.style) {
        style = {...style, ...col.style}
      }
      children = (
        <CardCellComponent data={record} cards={config} elements={col.elements}/>
      )
    } else if (col.type === 'operation') {
      style.padding = '0px 5px'
      children = (
        <Button type="link" style={{color: 'rgb(255, 77, 79)', backgroundColor: 'transparent'}} onClick={() => MKEmitter.emit('delRecord', col.tableId, {...record})}>删除</Button>
      )
    }

    return (<td className={className} style={style}>{col.addable ? <PlusCircleOutlined onClick={() => MKEmitter.emit('addRecord', col.tableId, {...record})} className="mk-editable mk-plus"/> : null}{children}{col.delable ? <DeleteOutlined onClick={() => MKEmitter.emit('delRecord', col.tableId, {...record})} className="mk-editable mk-del"/> : null}</td>)
    // return (<td className={className} style={style}>{children}</td>)
  }
}

class BodyAllCell extends React.Component {
  state = {
    err: null
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props.record), fromJS(nextProps.record)) ||
      nextState.err !== this.state.err
  }

  componentDidMount () {
    MKEmitter.addListener('tdFocus', this.tdFocus)
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('tdFocus', this.tdFocus)
  }

  tdFocus = (id) => {
    const { col, record } = this.props

    if (id !== col.uuid + record.$$uuid) return
    this.focus()
  }

  enterPress = () => {
    const { col, record } = this.props

    setTimeout(() => {
      if (col.enter === '$next') {
        MKEmitter.emit('nextLine', col, record.$$uuid)
      } else if (col.enter === '$sub') {
        MKEmitter.emit('subLine', col, record)
      } else if (col.enter !== '$noAct') {
        MKEmitter.emit('tdFocus', col.enter + record.$$uuid)
      }
    }, 50)
  }

  focus = () => {
    const { col, record } = this.props

    if (col.editType === 'switch' || col.editType === 'select') {
      let node = document.getElementById(col.uuid + record.$$uuid)
      node && node.click()
    } else {
      let err = null
      let val = record[col.field] !== undefined ? record[col.field] : ''
  
      if (col.type === 'number') {
        val = +val
        if (isNaN(val)) {
          val = 0
        }
        if (typeof(col.max) === 'number' && val > col.max) {
          err = col.label + '最大为' + col.max
        } else if (typeof(col.min) === 'number' && val < col.min) {
          err = col.label + '最小为' + col.min
        }
      } else if (col.required === 'true' && !val) {
        err = '请填写' + col.label
      }
  
      this.setState({err}, () => {
        let node = document.getElementById(col.uuid + record.$$uuid)
        node && node.select()
      })
    }
  }
  
  onChange = (val) => {
    const { col, record } = this.props
    
    let err = null

    if (col.type === 'number') {
      val = +val
      if (isNaN(val)) {
        val = 0
      }
      if (typeof(col.max) === 'number' && val > col.max) {
        err = col.label + '最大为' + col.max
      } else if (typeof(col.min) === 'number' && val < col.min) {
        err = col.label + '最小为' + col.min
      }
    } else if (col.required === 'true' && !val) {
      err = '请填写' + col.label
    }
    this.setState({err})
    MKEmitter.emit('changeRecord', col.tableId, {...record, [col.field]: val})
  }

  onSwitchChange = (val, label) => {
    const { col, record } = this.props

    setTimeout(() => {
      if (col.enter === '$next') {
        MKEmitter.emit('nextLine', col, record.$$uuid)
      } else if (col.enter === '$sub') {
        MKEmitter.emit('subLine', col, record)
      } else if (col.enter !== '$noAct') {
        MKEmitter.emit('tdFocus', col.enter + record.$$uuid)
      }
    }, 50)

    let values = {}
    if (col.editField) {
      values[col.field] = label
      values[col.editField] = val
    } else {
      values[col.field] = val
    }

    MKEmitter.emit('changeRecord', col.tableId, {...record, ...values})
  }

  onSelectChange = (val, option) => {
    const { col, record } = this.props

    let values = {}
    let _option = col.options.filter(m => m.key === option.key)[0]

    if (_option) {
      if (col.linkSubField) {
        col.linkSubField.forEach(m => {
          values[m] = _option[m] !== undefined ? _option[m] : ''
        })
      }

      if (col.editField) {
        values[col.field] = _option.label
        values[col.editField] = val
      } else {
        values[col.field] = val
      }
    }

    setTimeout(() => {
      if (col.enter === '$next') {
        MKEmitter.emit('nextLine', col, record.$$uuid)
      } else if (col.enter === '$sub') {
        MKEmitter.emit('subLine', col, record)
      } else if (col.enter !== '$noAct') {
        MKEmitter.emit('tdFocus', col.enter + record.$$uuid)
      }
    }, 50)

    MKEmitter.emit('changeRecord', col.tableId, {...record, ...values})
  }

  render() {
    let { col, config, record, style, className } = this.props
    const { err } = this.state

    let children = null
    if (col.type === 'text') {
      if (col.editable === 'true') {
        let _value = ''
        if (col.editField) {
          _value = record[col.editField] !== undefined ? record[col.editField] : ''
        } else {
          _value = record[col.field] !== undefined ? record[col.field] : ''
        }
        
        if (!col.editType || col.editType === 'text') {
          children = (<>
            <Input id={col.uuid + record.$$uuid} defaultValue={_value} onChange={(e) => this.onChange(e.target.value)} onPressEnter={this.enterPress} onBlur={this.onBlur}/>
            {err ? <Tooltip title={err}><ExclamationCircleOutlined /></Tooltip> : null}
          </>)
        } else if (col.editType === 'switch') {
          children = (
            <CusSwitch config={col} autoFocus={false} defaultValue={_value} onChange={this.onSwitchChange} onBlur={() => {}}/>
          )
        } else {
          children = (<>
            <Select
              showSearch
              dropdownClassName="edit-table-dropdown"
              dropdownMatchSelectWidth={col.dropdown === 'fixed'}
              defaultValue={_value}
              id={col.uuid + record.$$uuid}
              filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
              onSelect={this.onSelectChange}
            >
              {col.options.map((item, i) => (<Select.Option key={item.key} disabled={item.$disabled} value={item.value}>{item.label}</Select.Option>))}
            </Select>
          </>)
        }
      } else {
        let content = ''
        if (record[col.field] !== undefined) {
          content = `${record[col.field]}`
        }

        if (content !== '') {
          if (col.textFormat === 'YYYY-MM-DD' && /^[1-9]\d{3}(-|\/)(0[1-9]|1[0-2])(-|\/)(0[1-9]|[1-2][0-9]|3[0-1])/.test(content)) {
            content = `${content.substr(0, 4)}-${content.substr(5, 2)}-${content.substr(8, 2)}`
          } else if (col.textFormat === 'YYYY-MM-DD HH:mm:ss' && /^[1-9]\d{3}(-|\/)(0[1-9]|1[0-2])(-|\/)(0[1-9]|[1-2][0-9]|3[0-1]).([0-1][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]/.test(content)) {
            content = `${content.substr(0, 4)}-${content.substr(5, 2)}-${content.substr(8, 2)} ${content.substr(11, 2)}:${content.substr(14, 2)}:${content.substr(17, 2)}`
          } else if (col.textFormat === 'encryption') {
            content = <span>{col.prefix || ''}<Encrypts value={content} />{col.postfix || ''}</span>
          }

          if (col.textFormat !== 'encryption') {
            content = (col.prefix || '') + content + (col.postfix || '')
          }
        }

        if (col.marks) {
          let mark = getMark(col.marks, record, style)

          style = mark.style

          if (mark.icon) {
            if (mark.position === 'front') {
              content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
            } else {
              content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
            }
          } else if (mark.innerStyle) {
            content = <span style={mark.innerStyle}>{content}</span>
          }
        }
        children = content
      }
    } else if (col.type === 'number') {
      if (col.editable === 'true') {
        let _value = record[col.field] !== undefined ? record[col.field] : ''
        children = (<>
          <InputNumber id={col.uuid + record.$$uuid} defaultValue={_value} onChange={(val) => this.onChange(val)} onPressEnter={this.enterPress}/>
          {err ? <Tooltip title={err}><ExclamationCircleOutlined /></Tooltip> : null}
        </>)
      } else {
        let content = ''
        try {
          content = parseFloat(record[col.field])
          if (isNaN(content)) {
            content = ''
          }
        } catch (e) {
          content = ''
        }

        if (content !== '') {
          if (col.round) {
            content = Math.round(content * col.round) / col.round
          }
          if (col.format === 'percent') {
            content = content * 100
          } else if (col.format === 'abs') {
            content = Math.abs(content)
          }
          if (col.round) {
            content = content.toFixed(col.decimal)
          }
    
          if (col.format === 'thdSeparator') {
            content = content.replace(/\d{1,3}(?=(\d{3})+(\.\d*)?$)/g, '$&,')
          }
    
          content = (col.prefix || '') + content + (col.postfix || '')
        }

        if (col.marks) {
          let mark = getMark(col.marks, record, style)

          style = mark.style

          if (mark.icon) {
            if (mark.position === 'front') {
              content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
            } else {
              content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
            }
          } else if (mark.innerStyle) {
            content = <span style={mark.innerStyle}>{content}</span>
          }
        }
        children = content
      }
    } else if (col.type === 'textarea') {
      let content = ''
      if (record[col.field] !== undefined) {
        content = `${record[col.field]}`
      }

      if (content) {
        content = (col.prefix || '') + content + (col.postfix || '')
      }

      children = (
        <div>
          {content ? <Paragraph copyable ellipsis={{ rows: 3, expandable: true }}>{content}</Paragraph> : null }
        </div>
      )
    } else if (col.type === 'formula') {
      let content = col.formula
      Object.keys(record).forEach(key => {
        let reg = new RegExp('@' + key + '@', 'ig')
        content = content.replace(reg, record[key])
      })

      if (col.eval !== 'false') {
        try {
          // eslint-disable-next-line
          content = eval(content)
        } catch (e) {
          content = ''
        }
      }

      content = content === undefined ? '' : content

      if (content !== '') {
        content = `${col.prefix || ''}${content}${col.postfix || ''}`

        if (col.eval === 'false') {
          content = content.replace(/\n/ig, '<br/>').replace(/\s/ig, '&nbsp;')
          content = <span dangerouslySetInnerHTML={{__html: content}}></span>
        }
      }

      if (col.marks) {
        let mark = getMark(col.marks, record, style)

        style = mark.style

        if (mark.icon) {
          if (mark.position === 'front') {
            content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
          } else {
            content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
          }
        } else if (mark.innerStyle) {
          content = <span style={mark.innerStyle}>{content}</span>
        }
      }

      children = content
    } else if (col.type === 'custom') {
      style.padding = '0px'
      if (col.style) {
        style = {...style, ...col.style}
      }

      children = (
        <CardCellComponent data={record} cards={config} elements={col.elements}/>
      )
    } else if (col.type === 'operation') {
      style.padding = '0px 5px'
      children = (
        <Button type="link" style={{color: 'rgb(255, 77, 79)', backgroundColor: 'transparent'}} onClick={() => MKEmitter.emit('delRecord', col.tableId, {...record})}>删除</Button>
      )
    }

    return (<td className={'editing_all_table_cell ' + className} style={style}>{col.addable ? <PlusCircleOutlined onClick={() => MKEmitter.emit('addRecord', col.tableId, {...record})} className="mk-editable mk-plus"/> : null}{children}{col.delable ? <DeleteOutlined onClick={() => MKEmitter.emit('delRecord', col.tableId, {...record})} className="mk-editable mk-del"/> : null}</td>)
  }
}

class NormalTable extends Component {
  static propTpyes = {
    statFValue: PropTypes.any,       // 合计字段数据
    MenuID: PropTypes.string,        // 菜单Id
    setting: PropTypes.object,       // 表格全局设置：tableType（表格是否可选、单选、多选）、actionfixed（按钮固定）
    columns: PropTypes.array,        // 表格列
    lineMarks: PropTypes.any,        // 行标记
    fields: PropTypes.array,         // 组件字段集
    BID: PropTypes.any,              // 主表ID
    total: PropTypes.any,            // 总数
    loading: PropTypes.bool,         // 表格加载中
    refreshdata: PropTypes.func,     // 表格中排序列、页码的变化时刷新
    changeLock: PropTypes.func,
    chgSelectData: PropTypes.func,
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    data: [],
    edData: [],
    edColumns: [],
    selectedRowKeys: [],  // 表格中选中行
    tableId: '',          // 表格ID
    pageIndex: 1,         // 初始页面索引
    pageSize: 10,         // 每页数据条数
    columns: null,        // 显示列
    forms: [],
    pickup: false,        // 收起未选择项
    orderfields: {},      // 排序id与field转换
    loading: false,
    editable: false,
    pageOptions: []
  }

  UNSAFE_componentWillMount () {
    const { setting, fields, columns } = this.props
    let orderfields = {}
    let initEditLine = null
    let edColumns = []

    let _columns = []
    let deForms = []
    let _forms = {}

    let getColumns = (cols) => {
      return cols.map(item => {
        let cell = null
  
        if (item.type === 'colspan') {
          cell = { title: item.label, align: item.Align }
          cell.children = getColumns(item.subcols)
        } else {
          if (item.editable === 'true') {
            _forms[item.field] = item
            if (!initEditLine) {
              initEditLine = item
            }
          }
    
          if (item.type === 'text' && item.editable === 'true' && item.editType === 'select' && item.resourceType === '1') {
            let _option = Utils.getSelectQueryOptions(item)
    
            if (window.GLOB.debugger === true || window.debugger === true) {
              console.info(_option.sql)
            }
    
            item.base_sql = window.btoa(window.encodeURIComponent(_option.sql))
            item.arr_field = _option.field
    
            deForms.push(item)
          }
    
          if (item.field) {
            orderfields[item.uuid] = item.field
          }

          cell = {
            align: item.Align,
            dataIndex: item.uuid,
            title: item.label,
            sorter: !!(item.field && item.IsSort === 'true'),
            width: item.Width || 120,
            $type: item.type,
            onCell: record => ({
              record,
              col: item,
              config: item.type === 'custom' || item.type === 'action' ? {setting, columns: fields} : null,
            })
          }
        }
  
        return cell
      })
    }
    _columns = getColumns(columns)

    let forms = []
    fields.forEach(item => {
      if (item.field === setting.primaryKey) return

      if (_forms[item.field]) {
        forms.push({..._forms[item.field], datatype: item.datatype})
      } else {
        forms.push(item)
      }
    })

    _columns.forEach(item => {
      if (item.$type === 'action') return

      let _copy = fromJS(item).toJS()
      _copy.sorter = false

      if (item.editable === 'true') {
        _copy.title = <span>{item.label}<EditOutlined className="system-color mk-edit-sign"/></span>
      }
      edColumns.push(_copy)
    })

    if (setting.delable !== 'false' && setting.operType !== 'buoyMode') {
      edColumns.push({
        align: 'center',
        dataIndex: 'mkoperation',
        title: '操作',
        sorter: false,
        width: 100,
        onCell: record => ({
          record,
          col: {type: 'operation', tableId: setting.tableId},
        })
      })
    }

    let size = (setting.pageSize || 10) + ''
    let pageOptions = ['10', '25', '50', '100', '500', '1000']

    if (!pageOptions.includes(size)) {
      pageOptions.push(size)
      pageOptions = pageOptions.sort((a, b) => a - b)
    }

    this.setState({
      forms,
      pageSize: setting.pageSize || 10,
      pageOptions,
      columns: _columns,
      edColumns,
      tableId: setting.tableId,
      orderfields,
      initEditLine,
      editable: setting.editable === 'true'
    }, () => {
      if (deForms.length > 0) {
        this.improveActionForm(deForms)
      }

      const element = document.getElementById(setting.tableId)
      element && element.style.setProperty('--mk-table-border-color', setting.borderColor || '#e8e8e8')
      element && element.style.setProperty('--mk-table-color', setting.color || 'rgba(0, 0, 0, 0.65)')
      element && element.style.setProperty('--mk-table-font-size', setting.fontSize || '14px')
      element && element.style.setProperty('--mk-table-font-weight', setting.fontWeight || 'normal')
    })
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    MKEmitter.addListener('subLine', this.subLine)
    MKEmitter.addListener('nextLine', this.nextLine)
    MKEmitter.addListener('addRecord', this.addLine)
    MKEmitter.addListener('delRecord', this.delRecord)
    MKEmitter.addListener('resetTable', this.resetTable)
    MKEmitter.addListener('transferData', this.transferData)
    MKEmitter.addListener('changeRecord', this.changeRecord)
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('subLine', this.subLine)
    MKEmitter.removeListener('nextLine', this.nextLine)
    MKEmitter.removeListener('addRecord', this.addLine)
    MKEmitter.removeListener('delRecord', this.delRecord)
    MKEmitter.removeListener('resetTable', this.resetTable)
    MKEmitter.removeListener('transferData', this.transferData)
    MKEmitter.removeListener('changeRecord', this.changeRecord)
  }

  transferData = (menuid, data, type) => {
    if (menuid !== this.props.MenuID) return

    if (type !== 'line') {
      this.setState({data: data || []})

      if (this.state.editable && !this.state.pickup) {
        setTimeout(() => {
          this.pickupChange()
        }, 200)
      }
    } else if (type === 'line' && data.$$uuid) {
      let _data = this.state.data.map(item => {
        if (item.$$uuid === data.$$uuid) {
          return data
        } else {
          return item
        }
      })
      let _edData = this.state.edData.map(item => {
        if (item.$$uuid === data.$$uuid) {
          return data
        } else {
          return item
        }
      })

      this.setState({edData: _edData, data: _data})
    }
    this.setState({editable: false})
  }

  improveActionForm = (deForms) => {
    const { BID, setting } = this.props

    let deffers = []
    let mainItems = []  // 云端或单点数据
    let localItems = [] // 本地数据
    let cache = setting.cache !== 'false'

    deForms.forEach(item => {
      if (item.database === 'sso') {
        mainItems.push(`select '${item.uuid}' as obj_name,'${item.arr_field}' as arr_field,'${item.base_sql}' as LText`)
      } else {
        localItems.push(`select '${item.uuid}' as obj_name,'${item.arr_field}' as arr_field,'${item.base_sql}' as LText`)
      }
    })

    // 本地请求
    let param = {
      func: 'sPC_Get_SelectedList',
      LText: localItems.join(' union all '),
      obj_name: '',
      arr_field: '',
      table_type: 'Y',
      BID: BID || ''
    }

    if (param.LText) {
      param.LText = Utils.formatOptions(param.LText)
      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt(param.LText, param.timestamp)

      deffers.push(
        new Promise(resolve => {
          Api.getSystemCacheConfig(param, cache).then(res => {
            if (!res.status) {
              notification.warning({
                top: 92,
                message: res.message,
                duration: 5
              })
            }
            resolve(res)
          })
        })
      )
    }

    // 系统请求
    let mainparam = {
      func: 'sPC_Get_SelectedList',
      LText: mainItems.join(' union all '),
      obj_name: '',
      arr_field: '',
      table_type: 'Y',
      BID: BID || ''
    }

    if (mainparam.LText) {
      mainparam.LText = Utils.formatOptions(mainparam.LText)
      mainparam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      mainparam.secretkey = Utils.encrypt(mainparam.LText, mainparam.timestamp)

      if (window.GLOB.mainSystemApi) {
        mainparam.rduri = window.GLOB.mainSystemApi
      }

      deffers.push(
        new Promise(resolve => {
          Api.getSystemCacheConfig(mainparam, cache).then(res => {
            if (!res.status) {
              notification.warning({
                top: 92,
                message: res.message,
                duration: 5
              })
            }
            resolve(res)
          })
        })
      )
    }

    Promise.all(deffers).then(response => {
      let result = {...response[0], ...(response[1] || {})}

      delete result.ErrCode
      delete result.ErrMesg
      delete result.message
      delete result.status

      this.resetFormList(result)
    })
  }

  resetFormList = (result) => {
    const { columns } = this.props
    const { edColumns } = this.state

    let _edColumns = []

    let reCols = {}
    columns.forEach(item => {
      if (item.resourceType === '1' && result[item.uuid] && result[item.uuid].length > 0) {
        let options = []
        let _map = new Map()
        let all = false
        result[item.uuid].forEach(cell => {
          let _cell = {key: Utils.getuuid()}

          _cell.value = cell[item.valueField]
          _cell.label = cell[item.valueText]

          if (!_cell.label && _cell.label !== 0) {
            if (!all) {
              _cell.label = '全部'
              all = true
            } else {
              return
            }
          }

          if (_map.has(_cell.value)) return
          _map.set(_cell.value, 0)

          if (item.linkSubField) {
            item.linkSubField.forEach(m => {
              _cell[m] = cell[m] === undefined ? '' : cell[m]
            })
          }

          if (item.disableField && cell[item.disableField] && /^true$/ig.test(cell[item.disableField])) {
            _cell.$disabled = true
          }
  
          options.push(_cell)
        })

        item.options = options

        reCols[item.uuid] = item
      }
    })

    _edColumns = edColumns.map(item => {
      if (reCols[item.dataIndex]) {
        item.onCell = record => ({
          record,
          col: reCols[item.dataIndex]
        })
      }

      return item
    })

    if (this.state.pickup) {
      this.setState({
        pickup: false
      }, () => {
        this.setState({pickup: true, edColumns: _edColumns})
      })
    } else {
      this.setState({edColumns: _edColumns})
    }
  }
  
  nextLine = (col, uuid) => {
    const { setting } = this.props
    const { edData, initEditLine, tableId } = this.state

    if (col.tableId !== tableId) return

    let index = edData.findIndex(item => item.$$uuid === uuid)
    let next = edData[index + 1] || null

    if (next && initEditLine) {
      MKEmitter.emit('tdFocus', initEditLine.uuid + next.$$uuid)
    } else if (setting.addable === 'true') {
      setTimeout(() => {
        this.plusLine()
      }, 10)
    } else if (edData[index]) {
      setTimeout(() => {
        this.subLine(col, edData[index])
      }, 10)
    }
  }

  subLine = (col, record) => {
    const { tableId, forms, edData } = this.state

    if (col && col.tableId !== tableId) return

    if (edData.filter(item => !item.$origin).length > 1) {
      setTimeout(() => {
        this.submit(edData)
      }, 10)
      return
    }

    setTimeout(() => {
      let item = fromJS(record).toJS()
      let line = []
      forms.forEach(col => {
        if (col.editable !== 'true' || item.$deleted) {
          if (col.type === 'number') {
            item[col.field] = +item[col.field]
            if (isNaN(item[col.field])) {
              item[col.field] = 0
            }
          } else {
            item[col.field] = item[col.field] !== undefined ? (item[col.field] + '') : ''
          }
          return
        }
        if (col.type === 'text') {
          let val = item[col.field] !== undefined ? (item[col.field] + '') : ''
          if (col.required === 'true' && !val) {
            line.push(`${col.label}不可为空`)
          }
          item[col.field] = val
        } else if (col.type === 'number') {
          let val = item[col.field]
          if (!val && val !== 0) {
            line.push(`${col.label}不可为空`)
            return
          }
          val = +val
          if (isNaN(val)) {
            line.push(`${col.label}数据格式错误`)
            return
          }

          val = +val.toFixed(col.decimal || 0)
          
          if (typeof(col.max) === 'number' && val > col.max) {
            line.push(`${col.label}不可大于${col.max}`)
          } else if (typeof(col.min) === 'number' && val < col.min) {
            line.push(`${col.label}不可小于${col.min}`)
          }

          item[col.field] = val
        }
      })

      let err = line.join('，')

      if (err) {
        notification.warning({
          top: 92,
          message: err,
          duration: 5
        })
      } else {
        this.submit([item], 'simple')
      }
    }, 10)
  }

  plusLine = () => {
    const { edData, forms, initEditLine } = this.state

    let item = {...edData[edData.length - 1]}

    item.$$uuid = Utils.getguid()
    item.$type = 'add'
    item.$Index = ''

    forms.forEach(col => {
      if (col.initval !== '$copy') {
        item[col.field] = col.initval
      }
      if (col.type === 'number') {
        item[col.field] = +item[col.field]
        if (isNaN(item[col.field])) {
          item[col.field] = 0
        }
      }
      if (item[col.field] === undefined) {
        item[col.field] = ''
      }
    })

    this.setState({edData: [...edData, item]}, () => {
      MKEmitter.emit('tdFocus', initEditLine.uuid + item.$$uuid)
    })
  }

  delRecord = (id, record) => {
    const { setting } = this.props
    const { tableId, edData } = this.state

    if (id !== tableId) return

    let _data = []

    if (record.$type === 'add') {
      _data = edData.filter(item => item.$$uuid !== record.$$uuid)
    } else {
      _data = edData.map(item => {
        if (item.$$uuid === record.$$uuid) {
          record.$deleted = true
          record.$origin = false
          record.$type = 'del'
          return record
        } else {
          return item
        }
      })

      if (setting.commit === 'simple' && record.$deleted) {
        this.subLine(null, record)
      }
    }

    this.setState({edData: _data})
  }

  changeRecord = (id, record) => {
    const { tableId } = this.state

    if (id !== tableId) return

    let _data = this.state.edData.map(item => {
      if (item.$$uuid === record.$$uuid) {
        record.$origin = false
        return record
      } else {
        return item
      }
    })

    this.setState({edData: _data})
  }

  addLine = (id, record) => {
    const { BID } = this.props
    const { edData, forms, tableId } = this.state

    if (id) {
      if (id !== tableId) return
      let _edData = fromJS(edData).toJS()
      let index = _edData.findIndex(item => record.$$uuid === item.$$uuid)

      let item = {}

      item.$$uuid = Utils.getguid()
      item.$type = 'add'
      item.$Index = ''
      item.$$BID = BID || ''
  
      forms.forEach(col => {
        if (col.initval !== '$copy') {
          item[col.field] = col.initval
        }
        if (col.type === 'number') {
          item[col.field] = +item[col.field]
          if (isNaN(item[col.field])) {
            item[col.field] = 0
          }
        }
  
        if (item[col.field] === undefined) {
          item[col.field] = ''
        }
      })

      _edData.splice(index, 0, item)

      this.setState({edData: _edData})
    } else {
      let item = {}
      if (edData.length > 0) {
        item = {...edData[edData.length - 1]}
        item.$$uuid = Utils.getguid()
        item.$type = 'add'
        item.$Index = ''
      } else {
        item.$$uuid = Utils.getguid()
        item.$type = 'add'
        item.$Index = ''
        item.$$BID = BID || ''
      }
  
      forms.forEach(col => {
        if (col.initval !== '$copy') {
          item[col.field] = col.initval
        }
        if (col.type === 'number') {
          item[col.field] = +item[col.field]
          if (isNaN(item[col.field])) {
            item[col.field] = 0
          }
        }
  
        if (item[col.field] === undefined) {
          item[col.field] = ''
        }
      })
  
      this.setState({edData: [...edData, item]})
    }
  }

  checkData = () => {
    const { edData, forms } = this.state

    if (edData.length === 0) {
      notification.warning({
        top: 92,
        message: '提交数据不可为空！',
        duration: 5
      })
      return
    }
    let err = ''
    let Index = 1
    let data = fromJS(edData).toJS().map(item => {
      let line = []
      forms.forEach(col => {
        if (col.editable !== 'true' || item.$deleted) {
          if (col.type === 'number') {
            item[col.field] = +item[col.field]
            if (isNaN(item[col.field])) {
              item[col.field] = 0
            }
          } else {
            item[col.field] = item[col.field] !== undefined ? (item[col.field] + '') : ''
          }
          return
        }
        if (col.type === 'text') {
          let val = item[col.field] !== undefined ? (item[col.field] + '') : ''
          if (col.required === 'true' && !val) {
            line.push(`${col.label}不可为空`)
          }
          item[col.field] = val
        } else if (col.type === 'number') {
          let val = item[col.field]
          if (!val && val !== 0) {
            line.push(`${col.label}不可为空`)
            return
          }
          val = +val
          if (isNaN(val)) {
            line.push(`${col.label}数据格式错误`)
            return
          }

          val = +val.toFixed(col.decimal || 0)
          
          if (typeof(col.max) === 'number' && val > col.max) {
            line.push(`${col.label}不可大于${col.max}`)
          } else if (typeof(col.min) === 'number' && val < col.min) {
            line.push(`${col.label}不可小于${col.min}`)
          }

          item[col.field] = val
        }
      })

      if (line.length > 0) {
        err += `第${Index}行：` + line.join('，') + '；'
      }
      if (!item.$deleted) {
        Index++
      }

      return item
    })

    if (err) {
      notification.warning({
        top: 92,
        message: err,
        duration: 5
      })
    } else {
      this.submit(data)
    }
  }

  submit = (data, type) => {
    const { submit, BID, setting } = this.props
    const { forms } = this.state

    if (type !== 'simple' && (setting.commit === 'change' || setting.commit === 'simple')) {
      data = data.filter(item => !item.$origin)
    }

    if (data.length === 0) {
      notification.warning({
        top: 92,
        message: '数据未修改，不可提交！',
        duration: 5
      })
      return
    }

    let result = getEditTableSql(submit, data, forms)

    let param = {
      excel_in: result.lines,
      BID: BID || ''
    }

    this.setState({
      loading: true
    })

    if (submit.intertype === 'system') { // 系统存储过程
      param.func = 'sPC_TableData_InUpDe'
      
      if (sessionStorage.getItem('dataM') === 'true') { // 数据权限
        result.sql = result.sql.replace(/\$@/ig, '/*')
        result.sql = result.sql.replace(/@\$/ig, '*/')
        result.bottom = result.bottom.replace(/\$@/ig, '/*')
        result.bottom = result.bottom.replace(/@\$/ig, '*/')
      } else {
        result.sql = result.sql.replace(/@\$|\$@/ig, '')
        result.bottom = result.bottom.replace(/@\$|\$@/ig, '')
      }
      
      param.excel_in_type = 'true'
      param.LText1 = Utils.formatOptions(result.insert)
      param.LText2 = Utils.formatOptions(result.bottom)
      param.LText = Utils.formatOptions(result.sql)
      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt(param.LText, param.timestamp)

      param.menuname = submit.logLabel

      if (window.GLOB.probation) {
        param.s_debug_type = 'Y'
      }

      Api.genericInterface(param).then((res) => {
        if (res.status) {
          if (type === 'simple') {
            this.updataLine(data[0])
            this.execSuccess(res, type)
          } else {
            this.execSuccess(res)
          }
        } else {
          this.execError(res)
        }
      }, () => {
        this.execError({})
      })
    } else if (submit.intertype === 'inner' && submit.innerFunc) { // 自定义存储过程
      param.func = submit.innerFunc

      Api.genericInterface(param).then((res) => {
        if (res.status) {
          if (type === 'simple') {
            this.updataLine(data[0])
            this.execSuccess(res, type)
          } else {
            this.execSuccess(res)
          }
        } else {
          this.execError(res)
        }
      }, () => {
        this.execError({})
      })
    }
  }

  updataLine = (item) => {
    if (item.$type === 'del') {
      let _data = this.state.edData.filter(m => m.$$uuid !== item.$$uuid)

      this.setState({edData: _data})
    } else {
      let _data = this.state.edData.map(m => {
        if (m.$$uuid === item.$$uuid) {
          item.$origin = true
          return item
        }
        return m
      })

      this.setState({edData: _data})
    }
    MKEmitter.emit('reloadData', this.props.MenuID, item.$$uuid, item)
  }

  execSuccess = (res, type) => {
    const { submit } = this.props

    if (res && res.ErrCode === 'S') { // 执行成功
      notification.success({
        top: 92,
        message: res.ErrMesg || this.state.dict['main.action.confirm.success'],
        duration: submit.stime ? submit.stime : 2
      })
    } else if (res && res.ErrCode === 'Y') { // 执行成功
      Modal.success({
        title: res.ErrMesg || this.state.dict['main.action.confirm.success']
      })
    } else if (res && res.ErrCode === '-1') { // 完成后不提示

    }

    this.setState({
      loading: false
    })

    if (type === 'simple') return

    if (submit.closetab === 'true') {
      MKEmitter.emit('popclose')
    }
    if (submit.execSuccess !== 'never') {
      this.repick()
      MKEmitter.emit('refreshByButtonResult', submit.$menuId, submit.execSuccess, submit)
    }
  }

  execError = (res) => {
    const { submit } = this.props

    if (res.ErrCode === 'E') {
      Modal.error({
        title: res.message || res.ErrMesg,
      })
    } else if (res.ErrCode === 'N') {
      notification.error({
        top: 92,
        message: res.message || res.ErrMesg,
        duration: submit.ntime ? submit.ntime : 10
      })
    } else if (res.ErrCode === 'F') {
      notification.error({
        className: 'notification-custom-error',
        top: 92,
        message: res.message || res.ErrMesg,
        duration: submit.ftime ? submit.ftime : 10
      })
    } else if (res.ErrCode === 'NM') {
      message.error(res.message || res.ErrMesg)
    }
    
    this.setState({
      loading: false
    })

    if (submit.execError !== 'never') {
      this.repick()
      MKEmitter.emit('refreshByButtonResult', submit.$menuId, submit.execError, submit)
    }
  }

  repick = () => {
    const { setting } = this.props
    const { data } = this.state

    if (setting.submittal === 'true') {
      this.setState({editable: true})
    }

    this.props.changeLock(false)
    this.setState({
      data: [],
      edData: [],
      pickup: false,
    }, () => {
      this.setState({
        data: data,
      })
    })
  }

  /**
   * 
   */
  onSelectChange = selectedRowKeys => {
    this.setState({ selectedRowKeys })

    let activeId = ''
    if (selectedRowKeys.length > 0) {
      activeId = selectedRowKeys.slice(-1)[0]
    }
    this.changedata(activeId)
    this.selectdata(selectedRowKeys)
  }

  /**
   * @description 点击整行，触发切换， 判断是否可选，单选或多选，进行对应操作
   */
  changeRow = (index) => {
    const { setting } = this.props

    if (!setting.tableType || this.state.pickup) return
    
    let newkeys = fromJS(this.state.selectedRowKeys).toJS()

    let activeId = ''
    if (setting.tableType === 'radio') {
      activeId = index
      newkeys = [index]
      this.setState({ selectedRowKeys: newkeys })
    } else {
      if (newkeys.includes(index)) {
        newkeys = newkeys.filter(item => item !== index)

        if (newkeys.length > 0) {
          activeId = newkeys.slice(-1)[0]
        }
      } else {
        activeId = index
        newkeys.push(index)
      }

      this.setState({ selectedRowKeys: newkeys })
    }

    this.changedata(activeId)
    this.selectdata(newkeys)
  }

  changedata = (id) => {
    const { MenuID } = this.props
    const { data } = this.state

    let _data = ''

    if (id) {
      _data = data.filter(item => item.$$uuid === id)[0] || ''
    }

    MKEmitter.emit('resetSelectLine', MenuID, id, _data)
  }

  selectdata = (keys) => {
    const { data } = this.state

    let _data = data.filter(item => keys.includes(item.$$uuid))

    this.props.chgSelectData(_data)
  }

  changeTable = (pagination, filters, sorter) => {
    const { orderfields } = this.state

    this.setState({
      pageIndex: pagination.current,
      pageSize: pagination.pageSize,
      selectedRowKeys: [],
    })

    sorter.field = orderfields[sorter.field] || ''

    this.props.refreshdata(pagination, filters, sorter)
  }

  resetTable = (id, repage) => {
    const { MenuID } = this.props

    if (id !== MenuID) return

    if (repage === 'false') {
      this.setState({
        selectedRowKeys: [],
      })
    } else {
      this.setState({
        pageIndex: 1,
        selectedRowKeys: [],
      })
    }
  }

  pickupChange = () => {
    const { submit, MenuID, setting } = this.props
    const { data } = this.state

    let pickup = !this.state.pickup

    if (!submit.sheet) {
      notification.warning({
        top: 92,
        message: '提交按钮尚未设置，不可编辑！',
        duration: 5
      })
      return
    }

    if (!pickup && this.state.edData.filter(item => !item.$origin).length > 0) {
      const _this = this
      confirm({
        title: '数据已修改，确定放弃保存吗？',
        onOk() {
          _this.setState({
            data: [],
            edData: [],
            pickup
          }, () => {
            _this.setState({
              data: data,
              edData: pickup ? fromJS(data).toJS() : []
            })
          })
        },
        onCancel() {}
      })
    } else {
      pickup && MKEmitter.emit('resetSelectLine', MenuID, '', '')
      pickup && this.props.chgSelectData([])
      let keys = this.state.selectedRowKeys
      this.setState({
        data: [],
        edData: [],
        selectedRowKeys: [],
        pickup,
        loading: false,
        editable: false
      }, () => {
        if (pickup && setting.tableType === 'checkbox' && keys.length > 0) {
          this.setState({
            data: data,
            edData: fromJS(data).toJS().filter(item => {
              item.$origin = false
              
              return keys.includes(item.$$uuid)
            })
          })
        } else {
          this.setState({
            data: data,
            edData: pickup ? fromJS(data).toJS() : []
          })
        }
      })

    }
    this.props.changeLock(pickup)
  }

  render() {
    const { setting, statFValue, lineMarks, submit } = this.props
    const { pickup, tableId, data, edData, columns, edColumns, loading, pageOptions, selectedRowKeys } = this.state

    const components = {
      body: {
        row: BodyRow,
        cell: setting.editType !== 'multi' || !pickup ? BodyCell : BodyAllCell
      }
    }

    // 数据收起时，过滤已选数据
    let _data = data
    let _columns = columns

    if (pickup) {
      _data = edData
      _data = _data.filter(item => !item.$deleted)
      _columns = edColumns
    }

    // 设置表格选择属性：单选、多选、不可选
    let rowSelection = null
    if (setting.tableType && !pickup) {
      rowSelection = {
        selectedRowKeys,
        type: (setting.tableType === 'radio') ? 'radio' : 'checkbox',
        onChange: this.onSelectChange
      }
    }

    let _pagination = false
    if (!pickup && setting.laypage !== 'false' && setting.laypage !== false) {
      _pagination = {
        current: this.state.pageIndex,
        pageSize: this.state.pageSize,
        pageSizeOptions: pageOptions,
        showSizeChanger: true,
        total: this.props.total || 0,
        showTotal: (total, range) => `${range[0]}-${range[1]} 共 ${total} 条`
      }
    }

    let _footer = ''

    if (!pickup && statFValue && statFValue.length > 0) {
      _footer = statFValue.map(f => `${f.label}(合计)：${f.value}`).join('；')
    }

    let height = setting.height || false

    return (
      <>
        {submit.hasAction && pickup ? <div className="edit-custom-table-leftbtn-wrap">
          <Button style={submit.style} onClick={() => setTimeout(() => {this.checkData()}, 10)} loading={loading} className="submit-table" type="link">提交</Button>
        </div> : null}
        <div className="edit-custom-table-btn-wrap" style={submit.wrapStyle}>
          {!submit.hasAction && pickup ? <Button style={submit.style} onClick={() => setTimeout(() => {this.checkData()}, 10)} loading={loading} className="submit-table" type="link">提交</Button> : null}
          {setting.switchable !== 'false' ? <Switch title="编辑" className="main-pickup" checkedChildren="开" unCheckedChildren="关" disabled={loading || this.props.loading} checked={pickup} onChange={this.pickupChange} /> : null}
        </div>
        <div className={`edit-custom-table ${pickup ? 'editable' : ''} ${setting.tableHeader || ''} ${setting.operType || ''} ${height ? 'fixed-height' : ''} ${setting.mode || ''} table-vertical-${setting.vertical || ''} mk-edit-${setting.editType || 'simple'}`} id={tableId}>
          <Table
            rowKey="$$uuid"
            components={components}
            // style={setting.style}
            size={setting.size || 'middle'}
            bordered={setting.bordered !== 'false'}
            rowSelection={rowSelection}
            columns={_columns}
            dataSource={_data}
            loading={this.props.loading}
            scroll={{ x: '100%', y: height }}
            onRow={(record, index) => {
              return {
                lineMarks,
                data: record,
                onClick: () => {this.changeRow(record.$$uuid)},
              }
            }}
            onChange={this.changeTable}
            pagination={_pagination}
          />
          {_footer ? <div className={'normal-table-footer ' + (_pagination ? 'pagination' : '')}>{_footer}</div> : null}
          {pickup && setting.addable === 'true' ? <Button className="mk-add-line" onClick={() => this.addLine()} disabled={this.props.loading} type="link"><PlusOutlined /></Button> : null}
          {pickup && _data.length > 10 ? <Button style={submit.style} onClick={() => setTimeout(() => {this.checkData()}, 10)} loading={loading} className="submit-footer-table" type="link">提交</Button> : null}
        </div>
      </>
    )
  }
}

export default NormalTable